@@ -2,7 +2,7 @@
  * Author: M. Asselstine <asselsm@gmail.com>
  * Date  : 05-08-2005
  * Description : Handle communications with flickr.com
- * 
+ *
  * Copyright 2005 by M. Asselstine
 
  * This program is free software; you can redistribute it
@@ -10,12 +10,12 @@
  * Public License as published by the Free Software Foundation;
  * either version 2, or (at your option)
  * any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * ============================================================ */
 #include "flickrcomm.h"
 
@@ -32,19 +32,18 @@
 #include <qwmatrix.h>
 #include <kimageio.h>
 #include <qstrlist.h>
-#include <qcstring.h> 
+#include <qcstring.h>
 #include <kmdcodec.h>
 #include <qtextstream.h>
 #include <qstringlist.h>
 #include <kapplication.h>
 
+#include "photo.h"
 #include "photolistview.h"
 
 #include <algorithm>
 
-/*!
-    \fn FlickrComm::FlickrComm(QObject *parent, const char *name)
- */
+
 FlickrComm::FlickrComm(QObject *parent, const char *name)
  : QObject(parent, name),
  m_APIKey("c0134cf226b1187e3d79e4e1be03d1bf"),
@@ -57,52 +56,41 @@ FlickrComm::FlickrComm(QObject *parent, const char *name)
     m_MD5Context = new KMD5("");
 }
 
-/*!
-    \fn FlickrComm::~FlickrComm()
- */
 FlickrComm::~FlickrComm()
 {
     delete m_MD5Context;
 }
 
-
-/*!
-    \fn FlickrComm::generateMD5(const ArgMap &)
- */
 QString FlickrComm::generateMD5(const ArgMap &args)
 {
     QString str;
-    ArgMap::ConstIterator it;    
-    
+    ArgMap::ConstIterator it;
+
     // create the string from our arguments
     for( it = args.constBegin(); it != args.constEnd(); ++it )
     {
         str += it.key() + it.data();
     }
-      
+
     // generate MD5, we have to make use of the QCString returned by the call
     // to QString::utf8 in order to properly account for unicode issues
     m_MD5Context->reset();
     m_MD5Context->update(m_publicKey);
     m_MD5Context->update(str.utf8());
-    
+
     return m_MD5Context->hexDigest().data();
 }
 
-
-/*!
-    \fn FlickrComm::assembleArgs(const ArgMap &)
- */
 QString FlickrComm::assembleArgs(const ArgMap &args)
 {
     QString result;
-    ArgMap::ConstIterator it;    
-    
+    ArgMap::ConstIterator it;
+
     // create the request URL  XXX=YYY&ZZZ=AAA...
     for( it = args.constBegin(); it != args.constEnd(); ++it )
     {
         if( !result.isEmpty() )
-        {  
+        {
             result += "&";
         }
         result += it.key() + "=" + it.data();
@@ -110,27 +98,23 @@ QString FlickrComm::assembleArgs(const ArgMap &args)
     return result;
 }
 
-
-/*!
-    \fn FlickrComm::validateHTTPResponse(const QString &)
- */
 QString FlickrComm::validateHTTPResponse(const QString &str)
 {
     QString err;
     QDomNode node;
     QDomElement root;
     QDomDocument doc("response");
-    
+
     // pump str into the XML document
     if( !doc.setContent(str) )
     {
         return i18n("Unrecognizable response from Flickr.com");
     }
-    
+
     // start at document root, first node
     root = doc.documentElement();
     node = root.firstChild();
-    
+
     // check for fail response
     if( root.attribute("stat", "fail") == "fail")
     {
@@ -150,134 +134,115 @@ QString FlickrComm::validateHTTPResponse(const QString &str)
     return err;
 }
 
-
-/*!
-    \fn FlickrComm::setUserToken(const QString &)
- */
 void FlickrComm::setUserToken(const QString &str)
 {
     m_userToken = str;
 }
 
-
-/*!
-    \fn FlickrComm::sendRequest(ArgMap)
- */
 KIO::TransferJob* FlickrComm::sendRequest(ArgMap &args)
 {
     QString url = "http://www.flickr.com/services/rest/?";
-        
+
     // setup request specific arguments
     args["api_key"] = m_APIKey;
     args.insert("api_sig", generateMD5(args));
-    
+
     // formulate complete URL
     url += assembleArgs(args);
-    
+
     // send request
     KIO::TransferJob *job = KIO::http_post(url, QByteArray(0), false);
     job->addMetaData("content-type", "Content-Type: application/x-www-form-urlencoded");
-    
+
     connect(job,SIGNAL(result(KIO::Job*)),this,SLOT(jobResult(KIO::Job*)));
     connect(job,SIGNAL(data(KIO::Job*,const QByteArray&)),this,SLOT(jobData(KIO::Job*,const QByteArray&)));
-    
+
     return job;
 }
 
-
-/*!
-    \fn FlickrComm::sendFROBRequest()
- */
 void FlickrComm::sendFROBRequest()
 {
-    ArgMap args;    
-    
+    ArgMap args;
+
     // setup args
     args["method"] = "flickr.auth.getFrob";
-    
+
     // remember request type and make request
     KIO::TransferJob* job = sendRequest(args);
     m_requests[job] = FROB_REQ;
 }
 
-
-/*!
-    \fn FlickrComm::sendTokenRequest(const QString &frob)
- */
 void FlickrComm::sendTokenRequest(const QString &frob)
 {
-    ArgMap args;    
-    
+    ArgMap args;
+
     // setup args
     args["method"]  = "flickr.auth.getToken";
     args["frob"]    = frob;
-    
+
     // remember request type and make request
     KIO::TransferJob* job = sendRequest(args);
     m_requests[job] = TOKEN_REQ;
 }
 
-
-/*!
-    \fn FlickrComm::sendPhotosetsRequest(const QString &user)
- */
-void FlickrComm::sendPhotosetsRequest(const QString &user)
+void FlickrComm::sendPhotosetsRequest(const QString &token, const QString &user)
 {
-    ArgMap args;    
-    
+    ArgMap args;
+
     // setup args
     args["method"]      = "flickr.photosets.getList";
     args["user_id"]     = user;
-    
+    args["auth_token"]  = token;
+
     // remember request type and make request
     KIO::TransferJob* job = sendRequest(args);
-    m_requests[job] = PHOTOSET_REQ;   
+    m_requests[job] = PHOTOSET_REQ;
 }
 
+void FlickrComm::sendLicensesRequest()
+{
+    ArgMap args;
+
+    // setup args
+    args["method"]      = "flickr.photos.licenses.getInfo";
+
+    // remember request type and make request
+    KIO::TransferJob* job = sendRequest(args);
+    m_requests[job] = LICENSES_REQ;
+}
 
-/*!
-    \fn FlickrComm::sendTagsRequest(const QString &token, const QString &user)
- */
 void FlickrComm::sendTagsRequest(const QString &token, const QString &user)
 {
-    ArgMap args;    
-    
+    ArgMap args;
+
     // setup args
     args["method"]      = "flickr.tags.getListUser";
     args["user_id"]     = user;
     args["auth_token"]  = token;
-    
+
     // remember request type and make request
     KIO::TransferJob* job = sendRequest(args);
     m_requests[job] = TAGS_REQ;
 }
 
-
-/*!
-    \fn FlickrComm::sendUpStatusRequest( const QString &token )
- */
 void FlickrComm::sendUpStatusRequest( const QString &token )
 {
     ArgMap args;
-    
+
     // setup args
     args["method"]        = "flickr.people.getUploadStatus";
     args["auth_token"]    = token;
-    
+
     // remember request type and make request
     KIO::TransferJob* job = sendRequest(args);
     m_requests[job] = STATUS_REQ;
 }
 
-
-/*!
-    \fn FlickrComm::prepareWebAuthentication(const QString &frob)
- */
 void FlickrComm::doWebAuthentication(const QString &frob)
 {
     ArgMap args;
     QString addr;
-    
+
     // setup browser address string
     addr = "http://flickr.com/services/auth/";
     args["api_key"] = "c0134cf226b1187e3d79e4e1be03d1bf";
@@ -286,16 +251,12 @@ void FlickrComm::doWebAuthentication(const QString &frob)
     args.insert("api_sig", generateMD5(args));
 
     addr += "?" + assembleArgs(args);
-    
+
     // Open the default browser with the address
     new KRun( KURL( addr ) );
 }
 
-
-/*!
-    \fn FlickrComm::sendPhoto(const QString &token, PhotoListViewItem *photo)
- */
-void FlickrComm::sendPhoto(const QString &token, PhotoListViewItem *photo)
+void FlickrComm::sendPhoto(const QString &token, Photo *photo)
 {
     ArgMap args;
     QString encoded;
@@ -305,24 +266,24 @@ void FlickrComm::sendPhoto(const QString &token, PhotoListViewItem *photo)
     QBuffer buffer(array);
     QString url = "http://www.flickr.com/services/upload/?";
     QString boundary = QString("-----") + KApplication::randomString(20);
-    
-    
+
+
     // setup form
     buffer.open(IO_WriteOnly);
     QTextStream textStream(&buffer);
     textStream.setEncoding(QTextStream::UnicodeUTF8);
-    
+
     // setup request specific arguments
     args["api_key"] = m_APIKey;
     textStream << dd << boundary << crlf;
     textStream << "Content-Disposition: form-data; name=\"api_key\"" << crlf << crlf;
     textStream << m_APIKey << crlf;
-     
+
     args["auth_token"] = token;
     textStream << dd << boundary << crlf;
     textStream << "Content-Disposition: form-data; name=\"auth_token\"" << crlf << crlf;
     textStream << token << crlf;
-    
+
     if( !photo->title().isEmpty() )
     {
         args["title"] = photo->title();
@@ -330,7 +291,7 @@ void FlickrComm::sendPhoto(const QString &token, PhotoListViewItem *photo)
         textStream << "Content-Disposition: form-data; name=\"title\"" << crlf << crlf;
         textStream << photo->title() << crlf;
     }
-    
+
     if( !photo->description().isEmpty() )
     {
         args["description"] = photo->description();
@@ -338,7 +299,7 @@ void FlickrComm::sendPhoto(const QString &token, PhotoListViewItem *photo)
         textStream <<"Content-Disposition: form-data; name=\"description\"" << crlf << crlf;
         textStream << photo->description() << crlf;
     }
-    
+
     if( photo->tags().count() > 0 )
     {
         args["tags"] = photo->tags().join(" ");
@@ -346,82 +307,86 @@ void FlickrComm::sendPhoto(const QString &token, PhotoListViewItem *photo)
         textStream << "Content-Disposition: form-data; name=\"tags\"" << crlf << crlf;
         textStream << photo->tags().join(" ") << crlf;
     }
-    
-    args["is_public"] = (photo->isPublic() ? "1" : "0");
+
+    args["is_public"] = (photo->exposed() ? "1" : "0");
     textStream << dd << boundary << crlf;
     textStream << "Content-Disposition: form-data; name=\"is_public\"" << crlf << crlf;
-    textStream << (photo->isPublic( ) ? "1" : "0") << crlf;
-    
-    args["is_family"] = (photo->isFamily() ? "1" : "0");
+    textStream << (photo->exposed( ) ? "1" : "0") << crlf;
+
+    args["is_family"] = (photo->family() ? "1" : "0");
     textStream << dd << boundary << crlf;
     textStream << "Content-Disposition: form-data; name=\"is_family\"" << crlf << crlf;
-    textStream << (photo->isFamily() ? "1" : "0") << crlf;
-    
-    args["is_friend"] = (photo->isFriends() ? "1" : "0");
+    textStream << (photo->family() ? "1" : "0") << crlf;
+
+    args["is_friend"] = (photo->friends() ? "1" : "0");
     textStream << dd << boundary << crlf;
     textStream << "Content-Disposition: form-data; name=\"is_friend\"" << crlf << crlf;
-    textStream << (photo->isFriends() ? "1" : "0") << crlf;
-    
+    textStream << (photo->friends() ? "1" : "0") << crlf;
+
     // We can now generate our MD5
     QString md5 = generateMD5(args);
-    
+
     textStream << dd << boundary << crlf;
     textStream << "Content-Disposition: form-data; name=\"api_sig\"" << crlf << crlf;
     textStream << md5 << crlf;
-    
+
     textStream << dd << boundary << crlf;
     textStream << "Content-Disposition: form-data; name=\"photo\"; filename=\"";
-    textStream << photo->url().path() << QCString( "\"" ) << crlf;
-    textStream << "Content-Type: " << KImageIO::mimeType(photo->url().path()) << crlf << crlf;
-        
+    textStream << photo->URL().path() << QCString( "\"" ) << crlf;
+    textStream << "Content-Type: " << KImageIO::mimeType(photo->URL().path()) << crlf << crlf;
+
     if( photo->size() == i18n("Original") && photo->rotation() == 0 )
     {
         // put the file data into the array
-        QFile photoFile(photo->url().path());
+        QFile photoFile(photo->URL().path());
         if ( !photoFile.open(IO_ReadOnly) )
             return;
         textStream.writeRawBytes(photoFile.readAll().data(), photoFile.size());
         photoFile.close();
     }
     else
-    {    
+    {
         QByteArray EXIFData;
         QStrList fmts = QImage::outputFormats();
-        QString fmt = QImage::imageFormat(photo->url().path());
-            
+        QString fmt = QImage::imageFormat(photo->URL().path());
+
         // copy the EXIF data header from the photo file
         if( fmt == "JPEG" && fmts.contains("JPEG") )
         {
-            QFile photoFile(photo->url().path());
+            QFile photoFile(photo->URL().path());
             EXIFData = getEXIFData(photoFile);
-            photoFile.close();        
+            photoFile.close();
         }
-        
+
         // open the image
-        QImage photoImage(photo->url().path());
-        
+        QImage photoImage(photo->URL().path());
+
         // size the image
-        if( photo->size() == i18n("Square 75x75") )
+        if( !photo->size().contains(i18n("Original")) )
         {
-            photoImage = photoImage.smoothScale(75, 75, QImage::ScaleFree);
+            int width(photo->size().section(' ', 1, 1).section('x', 0, 0).toInt());
+            int height(photo->size().section(' ', 1, 1).section('x', 1, 1).toInt());
+
+            // Preserve width or height if they are "74" (our indication to use original values)
+            if( width == 74 )
+            {
+                width = photoImage.width();
+            }
+            if( height == 74 )
+            {
+                height= photoImage.height();
+            }
+
+            if( width <= 100 && height <= 100 )
+            {
+                photoImage = photoImage.smoothScale(width, height, QImage::ScaleFree);
+            }
+            else
+            {
+                photoImage = photoImage.smoothScale(width, height, QImage::ScaleMin);
+            }
         }
-        else if( photo->size() == i18n("Thumb 75x100") )
-        {
-            photoImage = photoImage.smoothScale(75, 100, QImage::ScaleFree);
-        }
-        else if( photo->size() == i18n("Small 180x240") )
-        {
-            photoImage = photoImage.smoothScale(180, 240, QImage::ScaleMin);
-        }
-        else if( photo->size() == i18n("Medium 375x500") )
-        {
-            photoImage = photoImage.smoothScale(375, 500, QImage::ScaleMin);
-        }
-        else if( photo->size() == i18n("Large 768x1024") )
-        {
-            photoImage = photoImage.smoothScale(768, 1024, QImage::ScaleMin);
-        }
-        
+
         // rotate the image
         if( photo->rotation() != 0 )
         {
@@ -429,7 +394,7 @@ void FlickrComm::sendPhoto(const QString &token, PhotoListViewItem *photo)
             matrix = matrix.rotate(photo->rotation());
             photoImage = photoImage.xForm(matrix);
         }
-        
+
         // add image data to our form data, maintain format if possible
         if( fmts.contains(fmt) )
         {
@@ -447,31 +412,27 @@ void FlickrComm::sendPhoto(const QString &token, PhotoListViewItem *photo)
            photoImage.save(&buffer, "PNG");
         }
     }
-    
+
     // end form
     textStream << crlf << dd << boundary << dd << crlf << "\0";
     buffer.close();
-    
+
     // send request
     KIO::TransferJob *job = KIO::http_post(url, array, false);
     job->addMetaData("content-type", "Content-Type: multipart/form-data; boundary=" + boundary);
-        
+
     connect(job,SIGNAL(result(KIO::Job*)),this,SLOT(jobResult(KIO::Job*)));
     connect(job,SIGNAL(data(KIO::Job*,const QByteArray&)),this,SLOT(jobData(KIO::Job*,const QByteArray&)));
 
     m_requests[job] = FILE_UPLOAD;
 }
 
-
-/*!
-    \fn FlickrComm::jobResult(KIO::Job *job)
- */
 void FlickrComm::jobResult(KIO::Job *job)
 {
     QString err;
     KIO::TransferJob *transferJob = dynamic_cast<KIO::TransferJob*>(job);
 
-    
+
     // Get associated request type
     if( transferJob )
     {
@@ -483,7 +444,7 @@ void FlickrComm::jobResult(KIO::Job *job)
             m_incomingData.erase(transferJob);
             return;
         }
-        
+
         // notify user of specific error
         if( m_requests[transferJob] != NO_REQ && (err = validateHTTPResponse(m_incomingData[transferJob])) != "" )
         {
@@ -492,7 +453,7 @@ void FlickrComm::jobResult(KIO::Job *job)
             m_incomingData.erase(transferJob);
             return;
         }
-        
+
         // process the good response
         switch( m_requests[transferJob] )
         {
@@ -517,6 +478,9 @@ void FlickrComm::jobResult(KIO::Job *job)
             case CREATE_PHOTOSET_REQ:
                 hanldeCreatePhotosetResponse(m_incomingData[transferJob]);
                 break;
+            case LICENSES_REQ:
+                handleLicensesResponse(m_incomingData[transferJob]);
+                break;
             default:
                 break;
         };
@@ -525,14 +489,10 @@ void FlickrComm::jobResult(KIO::Job *job)
     }
 }
 
-
-/*!
-    \fn FlickrComm::jobData(KIO::Job *job, const QByteArray &data)
- */
 void FlickrComm::jobData(KIO::Job *job, const QByteArray &data)
 {
     KIO::TransferJob *transferJob = dynamic_cast<KIO::TransferJob*>(job);
-    
+
     if( transferJob && !data.isEmpty() )
     {
         // Add new data to end of the associated buffer
@@ -540,28 +500,24 @@ void FlickrComm::jobData(KIO::Job *job, const QByteArray &data)
     }
 }
 
-
-/*!
-    \fn FlickrComm::handleFrobResponse(const QString &str)
- */
 void FlickrComm::handleFrobResponse(const QString &str)
 {
     QString frob = "";
     QDomNode node;
     QDomElement root;
     QDomDocument doc("frobresponse");
-    
+
     // pump str into the XML document
     if( !doc.setContent(str) )
     {
         emit commError(i18n("Invalid response received from Flickr.com."));
         return;
     }
-    
+
     // start at document root, first node
     root = doc.documentElement();
     node = root.firstChild();
-    
+
     // go through each node, should only be one
     while( !node.isNull() )
     {
@@ -573,7 +529,7 @@ void FlickrComm::handleFrobResponse(const QString &str)
         }
         node = node.nextSibling();
     }
-    
+
     // move on to next step if we have our frob
     if( !frob.isEmpty() )
     {
@@ -585,10 +541,6 @@ void FlickrComm::handleFrobResponse(const QString &str)
     }
 }
 
-
-/*!
-    \fn FlickrComm::handleTokenResponse(const QString &str)
- */
 void FlickrComm::handleTokenResponse(const QString &str)
 {
     QString nsid;
@@ -598,18 +550,18 @@ void FlickrComm::handleTokenResponse(const QString &str)
     QDomElement root;
     QString username = i18n("Unknown");
     QDomDocument doc("tokenresponse");
-    
+
     // pump str into the XML document
     if( !doc.setContent(str) )
     {
         emit commError(i18n("Invalid response received from Flickr.com."));
         return;
     }
-    
+
     // start at document root, first node
     root = doc.documentElement();
     node = root.firstChild();
-    
+
     // go through each node, should only be one
     while( !node.isNull() )
     {
@@ -619,14 +571,14 @@ void FlickrComm::handleTokenResponse(const QString &str)
             QDomElement elem = node.toElement();
             token = elem.text();
         }
-        
+
         // we got our permission
         if( node.isElement() && node.nodeName() == "perms" )
         {
             QDomElement elem = node.toElement();
             perms = elem.text();
         }
-        
+
         // for the user
         if( node.isElement() && node.nodeName() == "user" )
         {
@@ -634,7 +586,7 @@ void FlickrComm::handleTokenResponse(const QString &str)
             username = elem.attribute("username", i18n("Unknown"));
             nsid = elem.attribute("nsid", "");
         }
-        
+
         // step down the node tree
         if( node.isElement() && node.nodeName() == "auth" )
             node = node.firstChild();
@@ -643,32 +595,28 @@ void FlickrComm::handleTokenResponse(const QString &str)
     }
 
     // add the user and make active
-    emit returnedToken(username, token, nsid); 
+    emit returnedToken(username, token, nsid);
 }
 
-
-/*!
-    \fn FlickrComm::handleTagsResponse(const QString &str)
- */
 void FlickrComm::handleTagsResponse(const QString &str)
 {
     QDomNode node;
     QStringList tags;
     QDomElement root;
     QDomDocument doc("tagsresponse");
-    
-    
+
+
     // pump str into the XML document
     if( !doc.setContent(str) )
     {
         emit commError(i18n("Invalid response received from Flickr.com."));
         return;
     }
-    
+
     // start at document root, first node
     root = doc.documentElement();
     node = root.firstChild();
-    
+
     // go through each node, should only be one
     while( !node.isNull() )
     {
@@ -676,14 +624,14 @@ void FlickrComm::handleTagsResponse(const QString &str)
         if( node.isElement() && node.nodeName() == "tag" )
         {
             QDomElement elem = node.toElement();
-            
+
             // put quotes around multi word tags
             if( elem.text().contains(QRegExp("\\s+")) )
                 tags += QString("\"" + elem.text() + "\"");
             else
                 tags += elem.text();
         }
-        
+
         // step down the node tree
         if( node.isElement() && (node.nodeName() == "who" || node.nodeName() == "tags") )
             node = node.firstChild();
@@ -692,13 +640,9 @@ void FlickrComm::handleTagsResponse(const QString &str)
     }
 
     // add the user and make active
-    emit returnedTags(tags); 
+    emit returnedTags(tags);
 }
 
-
-/*!
-    \fn FlickrComm::handleStatusResponse(const QString &str)
- */
 void FlickrComm::handleStatusResponse(const QString &str)
 {
     QString max;
@@ -707,18 +651,18 @@ void FlickrComm::handleStatusResponse(const QString &str)
     QString remaining;
     QDomElement root;
     QDomDocument doc("statusresponse");
-    
+
     // pump str into the XML document
     if( !doc.setContent(str) )
     {
         emit commError(i18n("Invalid response received from Flickr.com."));
         return;
     }
-    
+
     // start at document root, first node
     root = doc.documentElement();
     node = root.firstChild();
-    
+
     // go through each node, should only be one
     while( !node.isNull() )
     {
@@ -729,7 +673,7 @@ void FlickrComm::handleStatusResponse(const QString &str)
             max = elem.attribute("max", "");
             used = elem.attribute("used", "");
         }
-        
+
         // step down the node tree
         if( node.isElement() && node.nodeName() == "user" )
             node = node.firstChild();
@@ -741,10 +685,10 @@ void FlickrComm::handleStatusResponse(const QString &str)
     if( max != "" && used != "" )
     {
         float value;
-        
+
         // remaining in Bytes
         value = max.toFloat() - used.toFloat();
-        
+
         // make a nice string
         if( value > (1024.0 * 1024.0 * 1024.0) )
         {
@@ -764,39 +708,35 @@ void FlickrComm::handleStatusResponse(const QString &str)
         else
         {
             remaining = QString::number(value) + "Bytes";
-        }            
+        }
     }
     else
     {
         remaining = i18n("Unknown");
     }
-    
+
     // emit the users upload remaining
-    emit returnedUploadStatus(remaining);    
+    emit returnedUploadStatus(remaining);
 }
 
-
-/*!
-    \fn FlickrComm::handleUploadResponse(const QString &str)
- */
 void FlickrComm::handleUploadResponse(const QString &str)
 {
     QString id;
     QDomNode node;
     QDomElement root;
     QDomDocument doc("uploadresponse");
-    
+
     // pump str into the XML document
     if( !doc.setContent(str) )
     {
         emit commError(i18n("Invalid response received from Flickr.com."));
         return;
     }
-    
+
     // start at document root, first node
     root = doc.documentElement();
     node = root.firstChild();
-    
+
     while( !node.isNull( ) )
     {
         // we got our id, good to go
@@ -812,10 +752,6 @@ void FlickrComm::handleUploadResponse(const QString &str)
     emit returnedUploadedOK(id);
 }
 
-
-/*!
-    \fn FlickrComm::handlePhotosetResponse(const QString &str)
- */
 void FlickrComm::handlePhotosetResponse(const QString &str)
 {
     QString id;
@@ -823,40 +759,40 @@ void FlickrComm::handlePhotosetResponse(const QString &str)
     QDomElement root;
     QStringList setTitles;
     QDomDocument doc("photosetsresponse");
-    
-    
+
+
     // pump str into the XML document
     if( !doc.setContent(str) )
     {
         emit commError(i18n("Invalid response received from Flickr.com."));
         return;
     }
-    
+
     // Clear the current photoset data
     m_photoSets.clear();
-    
+
     // start at document root, first node
     root = doc.documentElement();
     node = root.firstChild();
-    
+
     // go through each node, should only be one
     while( !node.isNull() )
     {
         // we got a set, store the id and fish out the title
         if( node.isElement() && node.nodeName() == "photoset" )
         {
-            QDomElement elem = node.toElement();            
+            QDomElement elem = node.toElement();
             id = elem.attribute("id");
-            
+
             elem = elem.elementsByTagName("title").item(0).toElement();
             if( id != QString::null )
-            {        
+            {
                 setTitles.append(elem.text());
                 m_photoSets.insert(elem.text(), id);
             }
-            
+
         }
-         
+
         // step down the node tree
         if( node.isElement() && node.nodeName() == "photosets" )
             node = node.firstChild();
@@ -865,15 +801,12 @@ void FlickrComm::handlePhotosetResponse(const QString &str)
     }
 
     // sort the title alphabetically
-    setTitles.sort();
-    
-    // notify interested widgets that new set titles are available    
-    emit returnedPhotosets(setTitles, QString::null); 
+    //setTitles.sort();
+
+    // notify interested widgets that new set titles are available
+    emit returnedPhotosets(setTitles, QString::null);
 }
 
-/*!
-    \fn FlickrComm::hanldeCreatePhotosetResponse(const QString &str)
- */
 void FlickrComm::hanldeCreatePhotosetResponse(const QString &str)
 {
     QString id;
@@ -883,18 +816,18 @@ void FlickrComm::hanldeCreatePhotosetResponse(const QString &str)
     SetData::Iterator it;
     QStringList setTitles;
     QDomDocument doc("photosetsresponse");
-    
+
     // pump str into the XML document
     if( !doc.setContent(str) )
     {
         emit commError(i18n("Invalid response received from Flickr.com."));
         return;
     }
-    
+
     // start at document root, first node
     root = doc.documentElement();
     node = root.firstChild();
-    
+
     // extract the id from the XML document
     while( !node.isNull() )
     {
@@ -913,27 +846,75 @@ void FlickrComm::hanldeCreatePhotosetResponse(const QString &str)
     for( it = m_photoSets.begin(); it != m_photoSets.end(); ++it )
     {
         setTitles.append(it.key());
-            
+
         // Is this the set we are creating
         if( it.data() == QString::null )
         {
            newTitle = it.key();         // store as active set
            m_photoSets[newTitle] = id;  // store the id
         }
-    }    
-    
-    // notify interested widgets that new set titles are available    
-    emit returnedPhotosets(setTitles, newTitle); 
+    }
+
+    // notify interested widgets that new set titles are available
+    emit returnedPhotosets(setTitles, newTitle);
 }
 
+void FlickrComm::handleLicensesResponse(const QString& str)
+{
+    QString id;
+    QString name;
+    QDomNode node;
+    QDomElement root;
+    QStringList licenseNames;
+    QDomDocument doc("licensesresponse");
+
+    // pump str into the XML document
+    if( !doc.setContent(str) )
+    {
+        emit commError(i18n("Invalid response received from Flickr.com."));
+        return;
+    }
+
+    // Clear the current licenses data
+    m_licenseIDs.clear();
+
+    // start at document root, first node
+    root = doc.documentElement();
+    node = root.firstChild();
+
+    // go through each node, should only be one
+    while( !node.isNull() )
+    {
+        // we got a license, store the id and fish out the name
+        if( node.isElement() && node.nodeName() == "license" )
+        {
+            QDomElement elem = node.toElement();
+            id = elem.attribute("id");
+            name = elem.attribute("name");
+
+            if( id != QString::null && name != QString::null )
+            {
+                licenseNames.append(name);
+                m_licenseIDs.insert(name, id);
+            }
+
+        }
+
+        // step down the node tree
+        if( node.isElement() && node.nodeName() == "licenses" )
+            node = node.firstChild();
+        else
+            node = node.nextSibling();
+    }
+
+    // notify any interested objects of the set of available licenses
+    emit returnedLicenses(licenseNames);
+}
 
-/*!
-    \fn FlickrComm::createPhotoset(const QString &token, const QString &name, const QString &photoID)
- */
 void FlickrComm::createPhotoset(const QString &token, const QString &name, const QString &photoID)
 {
     ArgMap args;
-    
+
     // Setup the flickr call
     args["method"]              = "flickr.photosets.create";
     args["title"]               = name;
@@ -942,16 +923,12 @@ void FlickrComm::createPhotoset(const QString &token, const QString &name, const
 
     // Add the set name to the SetData
     m_photoSets[name] = QString::null;
-    
+
     // Send the request
     KIO::TransferJob* job = sendRequest(args);
     m_requests[job] = CREATE_PHOTOSET_REQ;
 }
 
-
-/*!
-    \fn FlickrComm::addPhoto2Photoset(const QString &token, const QString &photoset, const QString &photoID)
- */
 void FlickrComm::addPhoto2Photoset(const QString &token, const QString &photoset, const QString &photoID)
 {
     // check if the set has to be created
@@ -962,27 +939,38 @@ void FlickrComm::addPhoto2Photoset(const QString &token, const QString &photoset
     else    // otherwise just add the photo to the set
     {
         ArgMap args;
-        
+
         // Setup the flickr call
         args["method"]      = "flickr.photosets.addPhoto";
         args["photoset_id"] = m_photoSets[photoset];
         args["photo_id"]    = photoID;
         args["auth_token"]  = token;
-    
+
         // Send the request
         KIO::TransferJob* job = sendRequest(args);
         m_requests[job] = ADD_PHOTO2SET_REQ;
     }
 }
 
+void FlickrComm::setPhotoLicense(const QString& token, const QString& license, const QString& photoID)
+{
+    ArgMap args;
+
+    // Setup the flickr call
+    args["method"]      = "flickr.photos.licenses.setLicense";
+    args["license_id"] = m_licenseIDs[license];
+    args["photo_id"]    = photoID;
+    args["auth_token"]  = token;
+
+    // Send the request
+    KIO::TransferJob* job = sendRequest(args);
+    m_requests[job] = SET_LICENSE_REQ;
+}
 
-/*!
-    \fn FlickrComm::abortCurrentRequest()
- */
 void FlickrComm::abortCurrentRequest()
 {
     QMap<KIO::TransferJob*,ResponseType>::iterator iter;
-    
+
     for( iter = m_requests.begin(); iter != m_requests.end(); ++iter )
     {
         iter.key()->kill();
@@ -991,25 +979,21 @@ void FlickrComm::abortCurrentRequest()
     m_incomingData.clear();
 }
 
-
-/*!
-    \fn FlickrComm::getEXIFData(QFile &in)
- */
 QByteArray FlickrComm::getEXIFData(QFile &in)
 {
     char data[6];
     ulong dataLength;
     QByteArray array;
-    
+
     if ( !in.open(IO_ReadOnly) )
         return false;
-    
+
     // Based on data from the web EXIF data will be at the start of the JPEG
     // file. All JPEG files start with 0xFFD8. This might be followed by JFIF
     // marker (0xFFE0) and data and then possible EXIF marker (0xFFE1) if
     // there is EXIF data. The (next byte * 256) + next byte will give the length
     // from the start of the EXIF marker to the end of the EXIF data.
-        
+
     // First ensure this is a valid JPEG file. Previous checks should
     // call QImage::imageFormat() so this is really just confirmation.
     ulong len = in.readBlock(data, 6);
@@ -1021,27 +1005,27 @@ QByteArray FlickrComm::getEXIFData(QFile &in)
             // JFIF data is present move beyond this data, raw data length
             // includes the length bytes but not the marker bytes.
             dataLength = ((uchar)data[4] * 256) + (uchar)data[5] + 2;
-            
+
             // Skip ahead in the file to the next marker. Ignore the two
             // bytes that start the file so that the bytes of interest for
-            // the EXIF check are 2 and 3 whether JFIF data is present or not 
+            // the EXIF check are 2 and 3 whether JFIF data is present or not
             in.at(dataLength);
-            
+
             // read in next (marker + length)
             len = in.readBlock(data, 6);
             if( len != 6 )
                 return false;
         }
-        
+
         // check for EXIF marker
         if( (uchar)data[2] == 0xff && (uchar)data[3] == 0xe1 )
         {
             // EXIF data is present find out how big it is (includes marker)
             // Raw data length includes the length bytes but not the marker bytes.
             dataLength = ((uchar)data[4] * 256) + (uchar)data[5] + 2;
-            
+
             // Write to the return array all the EXIF data.
-            in.at(in.at()-4);                
+            in.at(in.at()-4);
             array.resize(dataLength);
             len = in.readBlock(array.data(), dataLength);
             if( len != (dataLength) )
@@ -1053,10 +1037,6 @@ QByteArray FlickrComm::getEXIFData(QFile &in)
     return array;
 }
 
-
-/*!
-    \fn FlickrComm::writePhotoWithEXIF(QTextStream &out, QImage &in, QByteArray &exif)
- */
 void FlickrComm::writePhotoWithEXIF(QTextStream &out, QImage &in, QByteArray &exif)
 {
     int num2Remove;
@@ -1064,7 +1044,7 @@ void FlickrComm::writePhotoWithEXIF(QTextStream &out, QImage &in, QByteArray &ex
     ulong dataLength;
     QByteArray array;
     QBuffer buffer(array);
-    
+
     // Save the image to the buffer
     buffer.open(IO_WriteOnly);
     in.save(&buffer, "JPEG");
@@ -1073,7 +1053,7 @@ void FlickrComm::writePhotoWithEXIF(QTextStream &out, QImage &in, QByteArray &ex
     // Write the JPEG standard first two bytes to the stream
     out.writeRawBytes(&array[0], 2);
     arrayPos = 2;
-    
+
     // Write JFIF data if present
     if( (uchar)array[2] == 0xFF && (uchar)array[3] == 0xE0 )
     {
@@ -1083,10 +1063,10 @@ void FlickrComm::writePhotoWithEXIF(QTextStream &out, QImage &in, QByteArray &ex
         out.writeRawBytes(&array[2], dataLength);
         arrayPos += dataLength;
     }
-    
+
     // Write our passed in EXIF data
     out.writeRawBytes(exif.data(), exif.size());
-    
+
     // Skip current EXIF data if present
     if( (uchar)array[arrayPos] == 0xFF && (uchar)array[arrayPos+1] == 0xE1 )
     {
@@ -1094,7 +1074,7 @@ void FlickrComm::writePhotoWithEXIF(QTextStream &out, QImage &in, QByteArray &ex
         // included but not the marker bytes which also want included
         arrayPos += ((uchar)array[num2Remove+2] * 256) + (uchar)array[num2Remove+3] + 2;
     }
-        
+
     // Write everything after the arrayPos bytes.
     out.writeRawBytes(&array[arrayPos], array.size() - arrayPos);
 }
