@@ -1,8 +1,8 @@
 /* ============================================================
  * Author: M. Asselstine <asselsm@gmail.com>
  * Date  : 05-08-2005
- * Description : 
- * 
+ * Description :
+ *
  * Copyright 2005 by M. Asselstine
 
  * This program is free software; you can redistribute it
@@ -10,12 +10,12 @@
  * Public License as published by the Free Software Foundation;
  * either version 2, or (at your option)
  * any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * ============================================================ */
 
 
@@ -24,9 +24,9 @@
 #include <kshortcut.h>
 #include <kstdaction.h>
 #include <kactioncollection.h>
- 
+
 #include "kflickrpart.h"
- 
+
 kflickrPart* kflickrPart::m_instance;
 
 kflickrPart::kflickrPart(QWidget *parentWidget, const char *widgetName,
@@ -43,25 +43,23 @@ kflickrPart::kflickrPart(QWidget *parentWidget, const char *widgetName,
 
     // notify the part that this is our internal widget
     setWidget(&m_widget);
-    
+
     // The big four actions
     new KAction(i18n("&Add"), "add", KShortcut("CTRL++"), &m_widget,
         SLOT(addPhotos()), actionCollection(), "add_photos");
     new KAction(i18n("&Remove"), "remove", KShortcut("CTRL+-"), &m_widget,
         SLOT(removePhotos()), actionCollection(), "remove_photos");
-    new KAction(i18n("&Edit"), "edit", KShortcut("CTRL+e"), &m_widget,
-        SLOT(onEditSelected()), actionCollection(), "edit_photos");
     new KAction(i18n("&Upload"), "up", KShortcut("CTRL+u"), &m_widget,
         SLOT(uploadPhotos()), actionCollection(), "upload_photos");
-        
+
     // Host application signal connections
     connect(&m_widget, SIGNAL(photoCount(int)), SLOT(newPhotoCount(int)));
     connect(&m_widget, SIGNAL(bandwidthValue(const QString& )),
-            SLOT(newBandwidthValue(const QString &))); 
-        
+            SLOT(newBandwidthValue(const QString &)));
+
     // set our XML-UI resource file
     setXMLFile("kflickrpart.rc");
-    
+
     QTimer::singleShot(0, this, SLOT(setupStatusBar()));
 }
 
@@ -89,7 +87,7 @@ void kflickrPart::setupStatusBar()
 bool kflickrPart::openFile()
 {
     m_widget.addPhoto(m_url);
-    
+
     return true;
 }
 
@@ -143,7 +141,7 @@ KInstance* kflickrPartFactory::instance()
 {
     if( !s_instance )
     {
-        s_about = new KAboutData("kflickrpart", I18N_NOOP("kflickrPart"), "0.7");
+        s_about = new KAboutData("kflickrpart", I18N_NOOP("kflickrPart"), "0.8");
         s_about->addAuthor("M. Asselstine", 0, "asselsm@gmail.com");
         s_instance = new KInstance(s_about);
     }
