@@ -2,20 +2,20 @@
  * Author: M. Asselstine <asselsm@gmail.com>
  * Date  : 05-08-2005
  * Description : The main widget, show photo properties
- * 
+ *
  * Copyright 2005 by M. Asselstine
-
+ *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
  * Public License as published by the Free Software Foundation;
  * either version 2, or (at your option)
  * any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * ============================================================ */
 
 #include <kdialog.h>
@@ -27,7 +27,7 @@
 #include "kflickrwidget.h"
 #include "photolistview.h"
 #include "authcomplete.h"
-#include "photopropsdlg.h"
+#include "photoproperties.h"
 #include "authquestiondlg.h"
 
 #include <krun.h>
@@ -63,88 +63,77 @@ kflickrWidget::kflickrWidget(QWidget* parent, const char* name, WFlags fl)
     , m_uploadInProgress(false)
 {
     QValueList<int> widths;
-    
+
     // setup to retrieve saved settings
     KConfig *config = kapp->config();
-   
-    // config settings group 
+
+    // config settings group
     config->setGroup("KFlickrWidget");
-    
+
     // Stored user data
     m_users->insertStringList(config->readListEntry("users", QStringList()));
     m_userNSIDs= config->readListEntry( "user_nsids", QStringList() );
     m_userTokens = config->readListEntry( "user_tokens", QStringList() );
-    
+
+    m_previousBtn->setAccel(Key_PageUp);
+    m_previousBtn->setGuiItem(KGuiItem(i18n("Previous"),"up"));
+    m_nextBtn->setAccel(Key_PageDown);
+    m_nextBtn->setGuiItem(KGuiItem(i18n("Next"),"down"));
+
     // Set GUI icons
     m_addBtn->setGuiItem(KGuiItem(i18n("&Add"), "add"));
     m_removeBtn->setGuiItem(KGuiItem(i18n("&Remove"), "remove"));
-    m_editBtn->setGuiItem(KGuiItem(i18n("&Edit"), "edit"));
     m_uploadBtn->setGuiItem(KGuiItem(i18n("&Upload"), "up"));
-    
+
     // Photo list
-    widths = config->readIntListEntry( "column_widths" );
-    if( !widths.size() )
-    {
-        // default column widths
-        widths.append( 160 );
-        widths += 110;
-        widths += 150;
-        widths += 100;
-        widths += 225;
-        widths += 225;
-    }
-    widths[0] = 160;
-    
-    // initial list setup
     m_photolist = new PhotoListView( m_listHolder, "m_photolist" );
     QVBoxLayout* listLayout = new QVBoxLayout(m_listHolder, 0, 6);
     listLayout->addWidget(m_photolist);
-    
-    m_photolist->addColumn( i18n("Photo"), widths[0] );
-    m_photolist->addColumn( i18n("Public"), widths[1] );
-    m_photolist->addColumn( i18n("Title"), widths[2] );
-    m_photolist->addColumn( i18n("Size"), widths[3] );
-    m_photolist->addColumn( i18n("Tags"), widths[4] );
-    m_photolist->addColumn( i18n("Description"), widths[5] );
-   
+
+    m_photolist->addColumn( i18n("Photo") );
+
     m_photolist->header()->setResizeEnabled( false, 0 );
     m_photolist->setAllColumnsShowFocus( TRUE );
     m_photolist->setAcceptDrops( TRUE );
     m_photolist->setDropVisualizer( TRUE );
     m_photolist->setSelectionMode( QListView::Extended );
-        
+
+    // Photo Properties
+    m_photoProps = new PhotoProperties(m_propsHolder, "m_photoProps");
+    QVBoxLayout* propsLayout = new QVBoxLayout(m_propsHolder, 0, 6);
+    propsLayout->addWidget(m_photoProps);
+
     // widget signals and slots connections
     connect(m_addBtn, SIGNAL(clicked()), SLOT(addPhotos()));
     connect(m_removeBtn, SIGNAL(clicked()), SLOT(removePhotos()));
     connect(m_uploadBtn, SIGNAL(clicked()), SLOT(uploadPhotos()));
     connect(m_authNew, SIGNAL(clicked()), SLOT(authorizeNewUser()));
-    connect(m_editBtn, SIGNAL(clicked()), SLOT(onEditSelected()));
-    connect(this, SIGNAL(enableEdit(bool)), m_editBtn, SLOT(setEnabled(bool)));
-    connect(this, SIGNAL(enableRemove(bool)), m_removeBtn, SLOT(setEnabled(bool)));
-    connect(this, SIGNAL(enableUpload(bool)), m_uploadBtn, SLOT(setEnabled(bool)));
-    connect(m_photolist, SIGNAL(selectionChanged()), SLOT(updateActionStates()));
     connect(m_users, SIGNAL(activated(const QString &)), SLOT(setActiveUser(const QString &)));
-    connect(m_photolist, SIGNAL(dropped(QDropEvent*,QListViewItem*)),
-            SLOT(dropSlot(QDropEvent*,QListViewItem*)) );
+
+    connect(m_photolist, SIGNAL(selectionChanged()), SLOT(editSelected()));
+    connect(m_photolist, SIGNAL(selectionChanged()), SLOT(updateAvailableActions()));
+    connect(m_photolist, SIGNAL(dropped(QDropEvent*,QListViewItem*)), SLOT(dropSlot(QDropEvent*,QListViewItem*)) );
     connect(m_photolist, SIGNAL(itemAdded(QListViewItem*)), SLOT(updateCount(QListViewItem*)));
     connect(m_photolist, SIGNAL(itemRemoved(QListViewItem*)), SLOT(updateCount(QListViewItem*)));
 
-                
+    connect(m_nextBtn, SIGNAL(clicked()), m_photolist, SLOT(selectNext()));
+    connect(m_previousBtn, SIGNAL(clicked()), m_photolist, SLOT(selectPrevious()));
+
     // Communications signals and slots connections
-    connect(&m_comm, SIGNAL(commError(const QString &)), SLOT(handleCommError(const QString &)));
-    connect(&m_comm, SIGNAL(returnedFrob(const QString &)), SLOT(doUserAuthentication(const QString &)));
-    connect(&m_comm, SIGNAL(returnedToken(const QString &, const QString &, const QString &)),
-            SLOT(addUser(const QString &, const QString &, const QString &)));
-    connect(&m_comm, SIGNAL(returnedTags(const QStringList &)),
-            SLOT(setPersistentTags(const QStringList &)));
-    connect(&m_comm, SIGNAL(returnedPhotosets(const QStringList &, const QString &)),
-            SLOT(setPhotosets(const QStringList &, const QString &)));
-    connect(&m_comm, SIGNAL(returnedUploadStatus(const QString &)),
-            SLOT(newBandwidthValue(const QString &)));
-    connect(&m_comm, SIGNAL(returnedUploadedOK(const QString &)), SLOT(photoUploadedOK(const QString &)));
-    
-    connect(&m_timer, SIGNAL(timeout()), SLOT(refreshCount()));            
-    
+    connect(&m_comm, SIGNAL(commError(const QString&)), SLOT(handleCommError(const QString&)));
+    connect(&m_comm, SIGNAL(returnedFrob(const QString&)), SLOT(doUserAuthentication(const QString&)));
+    connect(&m_comm, SIGNAL(returnedToken(const QString&, const QString &, const QString&)), SLOT(addUser(const QString&, const QString&, const QString&)));
+    connect(&m_comm, SIGNAL(returnedTags(const QStringList&)), m_photoProps, SLOT(setAvailableTags(const QStringList &)));
+    connect(&m_comm, SIGNAL(returnedPhotosets(const QStringList&, const QString&)), m_photoProps, SLOT(setPhotosets(const QStringList&, const QString&)));
+    connect(&m_comm, SIGNAL(returnedUploadStatus(const QString&)), SLOT(newBandwidthValue(const QString&)));
+    connect(&m_comm, SIGNAL(returnedUploadedOK(const QString&)), SLOT(photoUploadedOK(const QString&)));
+    connect(&m_comm, SIGNAL(returnedLicenses(const QStringList&)), m_photoProps, SLOT(setLicenses(const QStringList&)));
+
+    connect(&m_timer, SIGNAL(timeout()), SLOT(refreshCount()));
+
+    // Update the available licenses by making a request to flickr
+    m_comm.sendLicensesRequest();
+
     // Finally set the active user if one already authorized
     if( config->readEntry( "current_user", QString( "" ) ) != "" )
         setActiveUser( config->readEntry( "current_user" ) );
@@ -156,133 +145,93 @@ kflickrWidget::~kflickrWidget()
     QValueList<int> widths;
     KConfig *config = kapp->config();
     QHeader *hdr = m_photolist->header();
-    
+
     // set the group
     config->setGroup("KFlickrWidget");
-    
+
     // save list column widths
     for( int i = 0; i < hdr->count(); ++i )
         widths.append( hdr->sectionSize( i ) );
     config->writeEntry( "column_widths", widths );
-    
+
     // save users
     for( int i = 0; i < m_users->count(); ++i )
         users += m_users->text( i );
     config->writeEntry( "users", users );
-    
+
     // save user tokens
     config->writeEntry( "user_nsids", m_userNSIDs );
     config->writeEntry( "user_tokens", m_userTokens );
-    
+
     // save current user
     config->writeEntry( "current_user", m_users->currentText() );
-    
+
     // delete stuff
     if( m_progressDlg != 0L )
         delete m_progressDlg;
 }
 
-
-/*!
-    \fn kflickrWidget::addPhoto(const KURL &url)
- */
- void kflickrWidget::addPhoto(const KURL &url)
- {
-    addPhoto(url, true);
- }
- 
- 
-/*!
-    \fn kflickrWidget::addPhoto(const KURL &url, bool doPreview)
- */
-PhotoListViewItem* kflickrWidget::addPhoto(const KURL &url, bool doPreview)
+void kflickrWidget::addPhoto(const KURL &url)
 {
-    // Create the photo in the list
-    PhotoListViewItem *photo = new PhotoListViewItem( m_photolist, url );
-    
-    // use the image filename as the default title
-    photo->setTitle( url.filename() );
-            
-    // get the previews
-    if( doPreview )
+    PhotoListViewItem *item = new PhotoListViewItem(m_photolist, url);
+
+    if( item )
     {
-        m_photolist->getPreview(url);
+        updateAvailableActions();
     }
-    
-    // update the button states
-    updateActionStates();
-    
-    return photo;
 }
 
-
-/*!
-    \fn kflickrWidget::addPhotos()
- */
-void kflickrWidget::addPhotos()
-{
-    // allow user to select some jpeg files to add
-    KURL::List urls =  KFileDialog::getOpenURLs( ":OpenPhoto", "*.jpg *.png *.gif|Photo Files",
-                                       this, i18n("Select Image(s)") );
-                                       
-    addPhotos(urls);    
-}
-
-
-/*!
-    \fn kflickrWidget::addPhotos(const KURL::List &urls)
- */
-void kflickrWidget::addPhotos(const KURL::List &urls)
-{
-    if( !urls.empty() )
-    {
-        KURL::List::const_iterator it;
-        
-        // add each selection to the photo list
-        for( it = urls.begin(); it != urls.end(); ++it )
-        {
-            addPhoto(*it, false);
-        }
-    }
-    
-    // get the previews
-    m_photolist->getPreviews(urls);
-}
-
-
-/*!
-    \fn kflickrWidget::addPhoto(const KURL &url,
-                                const QString &title,
-                                const QStringList &tags,
-                                const QString &desc)
- */
 void kflickrWidget::addPhoto(const KURL &url,
                              const QString &title,
                              const QStringList &tags,
                              const QString &desc)
 {
     // Add the photo to the list
-    PhotoListViewItem *photo = addPhoto(url, true);
-    
-    // Update the photo's attributes
-    if( title != QString::null )    photo->setTitle(title);
-    if( tags.count() > 0 )          photo->setTags(tags);
-    if( desc != QString::null )     photo->setDescription(desc);
+    PhotoListViewItem *item = new PhotoListViewItem(m_photolist, url);
+
+    if( item )
+    {
+        if( title != QString::null )    item->photo().title(title);
+        if( tags.count() > 0 )          item->photo().tags(tags);
+        if( desc != QString::null )     item->photo().description(desc);
+
+        updateAvailableActions();
+    }
 }
 
+void kflickrWidget::addPhotos()
+{
+    // allow user to select some jpeg files to add
+    KURL::List urls =  KFileDialog::getOpenURLs( ":OpenPhoto", "*.jpg *.png *.gif|Photo Files",
+                                       this, i18n("Select Image(s)") );
+
+    addPhotos(urls);
+}
+
+void kflickrWidget::addPhotos(const KURL::List &urls)
+{
+    if( !urls.empty() )
+    {
+        KURL::List::const_iterator it;
+
+        // add each selection to the photo list
+        for( it = urls.begin(); it != urls.end(); ++it )
+        {
+            addPhoto(*it);
+        }
+    }
+}
 
-/*!
-    \fn kflickrWidget::uploadPhotos()
- */
 void kflickrWidget::uploadPhotos()
 {
     if( m_photolist->childCount() > 0 )
     {
         m_uploadInProgress = true;
-        
+        m_photolist->clearSelection();
+
         // display the progress dialog
         showUploadProgress(m_photolist->childCount());
-        
+
         // upload the first photos one at a time
         uploadNextPhoto();
     }
@@ -294,14 +243,13 @@ void kflickrWidget::uploadPhotos()
  */
 void kflickrWidget::uploadNextPhoto()
 {
-    PhotoListViewItem *photo;
-    
     // get last photo in list
-    if( (photo = m_photolist->lastPhoto()) != 0L )
+    PhotoListViewItem *item = dynamic_cast<PhotoListViewItem*>(m_photolist->lastItem());
+
+    if( item )
     {
-        // upload the next photo
-        m_comm.sendPhoto(m_userTokens[m_currentUserIndex], photo);
-        updateUploadProgress(i18n("Sending Photo - %1").arg(photo->url().filename()));
+        m_comm.sendPhoto(m_userTokens[m_currentUserIndex], &(item->photo()));
+        updateUploadProgress(i18n("Sending Photo - %1").arg(item->photo().URL().filename()));
     }
     else
     {
@@ -310,6 +258,9 @@ void kflickrWidget::uploadNextPhoto()
         updateUploadProgress(i18n("Done"));
         m_uploadInProgress = false;
         hideUploadProgress();
+
+        m_photoProps->clearAndDisable();
+        updateAvailableActions();
     }
 }
 
@@ -319,25 +270,28 @@ void kflickrWidget::uploadNextPhoto()
  */
 void kflickrWidget::photoUploadedOK(const QString &photoID)
 {
-    PhotoListViewItem *photo;
-        
+
     // update unused bandwidth
-    m_comm.sendUpStatusRequest(m_userTokens[m_currentUserIndex]);    
-    
-    // remove the picture from the list
-    if( (photo = m_photolist->lastPhoto()) != 0L )
+    m_comm.sendUpStatusRequest(m_userTokens[m_currentUserIndex]);
+
+    // We know that the last picture uploaded was the last photograph in the list.
+    /// @todo Not actually true if the previous file failed to upload this may not be the case
+    PhotoListViewItem *item = dynamic_cast<PhotoListViewItem*>(m_photolist->lastItem());
+    if( item )
     {
-        QString photoset;
-        
         // Add photo to a photoset
-        if( (photoset = m_photosets->currentText()) != i18n("<photostream only>") )
+        if( item->photo().photoset() != i18n("<photostream only>") )
         {
-            m_comm.addPhoto2Photoset(m_userTokens[m_currentUserIndex], photoset, photoID);
+           m_comm.addPhoto2Photoset(m_userTokens[m_currentUserIndex], item->photo().photoset(), photoID);
         }
-        
-        delete photo;
+
+        // Set the license for the photograph
+        m_comm.setPhotoLicense(m_userTokens[m_currentUserIndex], item->photo().license(), photoID);
+
+        // Remove the photograph as it is now uploaded
+        delete item;
     }
-    
+
     // upload the next one
     uploadNextPhoto();
 }
@@ -360,13 +314,13 @@ void kflickrWidget::cancelUpload()
 void kflickrWidget::authorizeNewUser()
 {
     AuthQuestionDlg dlg( this );
-    
+
     // inform the user and check if we should continue
     if( dlg.exec() != QDialog::Accepted )
     {
         return;
     }
-    
+
     // Get our FROB from flickr.com
     m_comm.sendFROBRequest();
 }
@@ -378,53 +332,68 @@ void kflickrWidget::authorizeNewUser()
 void kflickrWidget::doUserAuthentication( const QString &frob )
 {
     AuthCompleteDlg dlg( this );
-    
+
     // Open browser etc... for web authentication
     m_comm.doWebAuthentication( frob );
-    
+
     // Wait for user to login and such at Flickr.com
     if( dlg.exec() != QDialog::Accepted )
     {
         return;
     }
-    
+
     // Request our TOKEN from flickr.com
     m_comm.sendTokenRequest( frob );
 }
 
-
-/*!
-    \fn kflickrWidget::newBandwidthValue(const QString &val)
- */
 void kflickrWidget::newBandwidthValue(const QString &val)
 {
     emit bandwidthValue(val);
 }
 
-
-/*!
-    \fn PhotoListView::updateCount(QListViewItem*)
- */
 void kflickrWidget::updateCount(QListViewItem*)
 {
     m_timer.start(500, true);
 }
 
-/*!
-    \fn PhotoListView::refreshCount()
- */
 void kflickrWidget::refreshCount()
 {
     emit photoCount(m_photolist->childCount());
 }
 
-/*!
-    \fn kflickrWidget::addUser( const QString &name, const QString &token )
- */
+void kflickrWidget::editSelected()
+{
+    // If we have a single photo selected populate the properties widget
+    // with the selected photo's properties. If we have more then one photo
+    // selected do a batch edit in the properties widget. If we have no
+    // photos selected disable the properties widget widgets.
+    if( m_photolist->numSelected() == 1 )
+    {
+        QListViewItemIterator it(m_photolist, QListViewItemIterator::Selected);
+        m_photoProps->editSinglePhoto(static_cast<PhotoListViewItem*>(it.current())->photo());
+    }
+    else if( m_photolist->numSelected() > 1 ) 
+    {
+        QPtrList<Photo> photographs;
+        QPtrList<QListViewItem> selected = m_photolist->selectedItems();
+        PhotoListViewItem* item = static_cast<PhotoListViewItem*>(selected.first());
+        while( item )
+        {
+            photographs.append(&(item->photo()));
+            item = static_cast<PhotoListViewItem*>(selected.next());
+        }
+        m_photoProps->editPhotoBatch(photographs);
+    }
+    else 
+    {
+        m_photoProps->clearAndDisable();
+    }
+}
+
 void kflickrWidget::addUser( const QString &name, const QString &token, const QString &nsid )
 {
     int i;
-    
+
     // check if the user already exists
     for( i = 0; i < m_users->count(); ++i )
     {
@@ -433,18 +402,18 @@ void kflickrWidget::addUser( const QString &name, const QString &token, const QS
             break;
         }
     }
-    
+
     // brand new user, existing users retain their current token
     if( i == m_users->count() )
     {
         m_userNSIDs.append( nsid );
         m_users->insertItem( name );
-        m_userTokens.append( token );        
+        m_userTokens.append( token );
     }
-    
+
     // make the active user
     setActiveUser( name );
-    
+
     // Notify user
     QMessageBox::information( this, i18n("New User"),
                 i18n("A new user '%1' has been added successfully.").arg(name) );
@@ -459,31 +428,30 @@ void kflickrWidget::setActiveUser( const QString &name )
     // update combobox
     if( m_users->currentText() != name )
         m_users->setCurrentText( name );
-        
+
     // update current user index, must always do this here
     m_currentUserIndex = m_users->currentItem();
-    
+
     // request users tags
     m_comm.sendTagsRequest(m_userTokens[m_currentUserIndex], m_userNSIDs[m_currentUserIndex]);
-    
+
     // request user photosets
-    m_comm.sendPhotosetsRequest(m_userNSIDs[m_currentUserIndex]);
-    
+    m_comm.sendPhotosetsRequest(m_userTokens[m_currentUserIndex], m_userNSIDs[m_currentUserIndex]);
+
     // update unused bandwidth
     m_comm.sendUpStatusRequest(m_userTokens[m_currentUserIndex]);
-    
-    
+
     // update the button states
-    updateActionStates();
+    updateAvailableActions();
 }
 
 void kflickrWidget::removePhotos()
 {
     // remove selected photos
     m_photolist->removeSelected();
-    
+
     // update the button states
-    updateActionStates();
+    updateAvailableActions();
 }
 
 
@@ -493,168 +461,37 @@ void kflickrWidget::removePhotos()
 void kflickrWidget::dropSlot( QDropEvent *e, QListViewItem *after )
 {
     QString text;
-    KURL::List urls;
     QStringList parts;
-    
+    PhotoListViewItem* afterItem = dynamic_cast<PhotoListViewItem*>(after);
+
     // make sure the drag is of the right type
     if ( QTextDrag::decode( e, text ) )
     {
         // there might be more than one file, split it up
         parts = QStringList::split( "\n", text );
-        
+
         for ( QStringList::Iterator it = parts.begin(); it != parts.end(); ++it )
         {
             // turn into a URL
             KURL url( (*it).stripWhiteSpace() );
-            
+
             // ensure it is a jpeg type file
             if( (url.protocol() == "file") &&
                 ( (url.filename()).endsWith( ".jpg", false ) ||
                   (url.filename()).endsWith( ".png", false ) ||
                   (url.filename()).endsWith( ".gif", false ) ) )
             {
-                // add it to the photo list
-                PhotoListViewItem *photo = new PhotoListViewItem( m_photolist, url, after );
-                
-                // use the filename as the default title
-                photo->setTitle( url.filename() );
-                
-                // add it for preview
-                urls.append(url);
-             }
-        }
-    }
-    
-    // get the previews
-    if(!urls.isEmpty())
-        m_photolist->getPreviews(urls);
-        
-    // update the button states
-    updateActionStates();
-}
-
-
-/*!
-    \fn kflickrWidget::onEditSelected()
- */
-void kflickrWidget::onEditSelected()
-{
-    if( m_photolist->numSelected() == 1 )
-        editSelectedItem();
-    else
-        editSelectedItems();
-}
-
-
-/*!
-    \fn kflickrWidget::editSelectedItems()
- */
-void kflickrWidget::editSelectedItems()
-{
-    PhotoPropsDlg* dlg = new PhotoPropsDlg(this);    
-    
-    // Setup the dialog
-    dlg->editPhotoBatch();
-    dlg->setAvailableTags(m_persistTags);
-    
-    // handle an "OK" response from the dialog
-    if( dlg->exec() == QDialog::Accepted )
-    {
-        PhotoListViewItem *photo;
-                
-        // starting with the first selected photo,
-        // edit the selected photos attributes
-        photo = m_photolist->firstSelectedPhoto();
-        while( photo != 0L )
-        {
-            if(!dlg->title().isEmpty())         photo->setTitle(dlg->title());
-            if(!dlg->description().isEmpty())   photo->setDescription(dlg->description());
-            if(!dlg->tags().empty())            photo->setTags(dlg->tags());
-            if(!dlg->photoSize().isEmpty())     photo->setSize(dlg->photoSize());
-            
-            // Only update privacy if user has selected a privacy level
-            if( dlg->isPublic() || dlg->isFamily() || dlg->isFriends() || dlg->isPrivate() )
-            {
-                photo->setPublic(dlg->isPublic());
-                photo->setFamily(dlg->isFamily());
-                photo->setFriends(dlg->isFriends());
+                // If afterItem is null we are dropping at the start of the list or in an empty
+                // list. This will result in the dropped item appearing first in the list.
+                new PhotoListViewItem(m_photolist, url, afterItem);
             }
-            
-            // move on to next selected photo
-            photo = m_photolist->nextSelectedPhoto(photo);
-        }    
-        m_persistTags = dlg->availableTags();
-    }
-    delete dlg;   
-}
-
-
-/*!
-    \fn kflickrWidget::editSelectedItem()
- */
-void kflickrWidget::editSelectedItem()
-{
-    PhotoListViewItem *photo;
-        
-    if( (photo = m_photolist->firstSelectedPhoto()) != 0L )
-    {
-        PhotoPropsDlg* dlg = new PhotoPropsDlg( this );
-        
-        
-        // setup the dialog
-        dlg->editSinglePhoto(photo);
-        dlg->setAvailableTags(m_persistTags);
-        
-        // handle an 'OK' response in the dialog
-        if( dlg->exec() == QDialog::Accepted )
-        {
-            dlg->updateActivePhotoInfo();
-            m_persistTags = dlg->availableTags();
         }
-        delete dlg;   
     }
+
+    // update the button states
+    updateAvailableActions();
 }
 
-
-/*
-    \fn kflickrWidget::persistentTags() const
- */
-QStringList kflickrWidget::persistentTags() const
-{
-    return m_persistTags;
-}
-
-
-/*!
-    \fn kflickrWidget::setPersistentTags(const QStringList &lst)
- */
-void kflickrWidget::setPersistentTags(const QStringList &lst)
-{
-    m_persistTags = lst;
-}
-
-
-/*!
-    \fn kflickrWidget::setPhotosets(const QStringList &titles, const QString &sel)
- */
-void kflickrWidget::setPhotosets(const QStringList &titles, const QString &sel)
-{
-    // update the photosets combobox
-    m_photosets->clear();
-    m_photosets->insertStringList(titles);
-    m_photosets->insertItem(i18n("<photostream only>"), 0);
-    
-    if( sel == QString::null )
-        m_photosets->setCurrentText(i18n("<photostream only>"));
-    else
-        m_photosets->setCurrentText(sel);
-        
-}
-
-
-/*!
-    \fn kflickrWidget::hideUploadProgress()
- */
 void kflickrWidget::hideUploadProgress()
 {
     // hide the progress dialog
@@ -664,53 +501,41 @@ void kflickrWidget::hideUploadProgress()
     }
 }
 
-
-/*!
-    \fn kflickrWidget::showUploadProgress(int numPhotos)
- */
 void kflickrWidget::showUploadProgress(int numPhotos)
 {
-    // create the progress dialog if it does not yet exist    
+    // create the progress dialog if it does not yet exist
     if( m_progressDlg == 0L )
     {
-        m_progressDlg = new KProgressDialog(this, "m_progressDlg");    
+        m_progressDlg = new KProgressDialog(this, "m_progressDlg");
         m_progressDlg->progressBar()->setTextEnabled(false);
         m_progressDlg->setCaption(i18n("Upload Progress"));
         m_progressDlg->setAutoClose(false);
         m_progressDlg->setModal(true);
-            
+
         // Progress dialog connections
         connect(m_progressDlg, SIGNAL(cancelClicked()), this, SLOT(cancelUpload()));
     }
-    
+
     // tune the progress dialog accordingly
     m_progressDlg->progressBar()->setTotalSteps(numPhotos);
     m_progressDlg->progressBar()->setValue(0);
     m_progressDlg->show();
 }
 
-
-/*!
-    \fn kflickrWidget::updateUploadProgress(const QString &str)
- */
 void kflickrWidget::updateUploadProgress(const QString &str)
 {
     // set text
     m_progressDlg->setLabel(str);
-    
+
     // move progress bar
     if( m_progressDlg->progressBar()->totalSteps() != m_photolist->childCount())
         m_progressDlg->progressBar()->advance(1);
 }
 
-
-/*!
-    \fn kflickrWidget::handleCommError(const QString &)
- */
 void kflickrWidget::handleCommError(const QString &errstr)
 {
     QMessageBox::critical(this, i18n("Error"), errstr);
-    
+
     // Are we in the middle of an upload
     if( m_uploadInProgress )
     {
@@ -726,32 +551,36 @@ void kflickrWidget::handleCommError(const QString &errstr)
     }
 }
 
-
-/*!
-    \fn kflickrWidget::updateActionStates()
- */
-void kflickrWidget::updateActionStates()
+void kflickrWidget::updateAvailableActions()
 {
     // Upload button enabled if we have a valid user and at least one photo
-    if( m_photolist->firstChild() != 0L && m_currentUserIndex != -1 )
+    m_uploadBtn->setEnabled(FALSE);
+    if( m_photolist->firstChild() && m_currentUserIndex != -1 )
     {
-        emit enableUpload(true);
+        m_uploadBtn->setEnabled(TRUE);
     }
-    else
-    {
-        emit enableUpload(false);
-    }
-    
-    // remove and edit only available if at least one photo selected
+
+    // remove action is only available if at least one photo selected
+    m_removeBtn->setEnabled(FALSE);
     if( m_photolist->numSelected() > 0 )
     {
-        emit enableRemove(true);
-        emit enableEdit(true);
+        m_removeBtn->setEnabled(TRUE);
     }
-    else
+
+    // enable next and previous buttons according to the selection
+    m_nextBtn->setEnabled(FALSE);
+    m_previousBtn->setEnabled(FALSE);
+    if( m_photolist->numSelected() == 1 )
     {
-        emit enableRemove(false);
-        emit enableEdit(false);
+        QListViewItem* item = QListViewItemIterator(m_photolist, QListViewItemIterator::Selected).current();
+        if( item != m_photolist->firstChild() )
+        {
+            m_previousBtn->setEnabled(TRUE);
+        }
+        if( item != m_photolist->lastItem() )
+        {
+            m_nextBtn->setEnabled(TRUE);
+        }
     }
 }
 
