@@ -2,7 +2,7 @@
  * Author: M. Asselstine <asselsm@gmail.com>
  * Date  : 05-08-2005
  * Description : The main widget, show photo properties
- * 
+ *
  * Copyright 2005 by M. Asselstine
 
  * This program is free software; you can redistribute it
@@ -10,19 +10,21 @@
  * Public License as published by the Free Software Foundation;
  * either version 2, or (at your option)
  * any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * ============================================================ */
 
 
 #ifndef _KFLICKRWIDGET_H_
 #define _KFLICKRWIDGET_H_
 
+#include "photo.h"
 #include "flickrcomm.h"
+#include "photoproperties.h"
 #include "kflickrWidgetUI.h"
 #include "kflickrwidgetiface.h"
 
@@ -54,44 +56,25 @@ class kflickrWidget : public kflickrWidgetUI, virtual public kflickrWidgetIface
 public:
     kflickrWidget(QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
     ~kflickrWidget();
-    
-    QStringList persistentTags() const;
-    
+
     void addPhoto(const KURL &url);
-    PhotoListViewItem* addPhoto(const KURL &url, bool doPreview);
+    void addPhoto(const KURL &url, const QString &title, const QStringList &tags, const QString &desc);
     void addPhotos(const KURL::List &urls);
-    void addPhoto(const KURL &url, const QString &title,
-                  const QStringList &tags, const QString &desc);
 
 public slots:
     void addPhotos();
     void removePhotos();
     void uploadPhotos();
-    void onEditSelected();
-    void updateActionStates();
+    void updateAvailableActions();
     void updateCount(QListViewItem*);
-    
+
     void handleCommError(const QString &);
     void dropSlot(QDropEvent*, QListViewItem*);
-    void setPersistentTags(const QStringList &);
     void doUserAuthentication(const QString &frob);
     void newBandwidthValue(const QString &val);
-    void setPhotosets(const QStringList &, const QString &);
     void addUser(const QString &name, const QString &token, const QString &nsid);
 
 private:
-    int m_currentUserIndex;
-    QStringList m_userNSIDs;
-    QStringList m_userTokens;
-    PhotoListView* m_photolist;
-
-    QTimer m_timer;
-    FlickrComm m_comm;
-    QStringList m_persistTags;
-    KProgressDialog *m_progressDlg;
-    bool m_uploadInProgress;
-
-private:        
     void uploadNextPhoto();
     void hideUploadProgress();
     void showUploadProgress(int numPhotos);
@@ -100,21 +83,27 @@ private:
 private slots:
     void cancelUpload();
     void authorizeNewUser();
-    
-    void editSelectedItem();
-    void editSelectedItems();
-    
+
     void refreshCount();
+    void editSelected();
     void setActiveUser(const QString &);
     void photoUploadedOK(const QString &);
-    
+
 signals:
-    void enableEdit(bool);
-    void enableRemove(bool);
-    void enableUpload(bool);
-    
     void photoCount(int);
     void bandwidthValue(const QString&);
+
+private:
+    int m_currentUserIndex;
+    QStringList m_userNSIDs;
+    QStringList m_userTokens;
+    PhotoListView* m_photolist;
+    PhotoProperties* m_photoProps;
+
+    QTimer m_timer;
+    FlickrComm m_comm;
+    KProgressDialog *m_progressDlg;
+    bool m_uploadInProgress;
 };
 
 #endif
